package arm_test

import (
	"testing"

	"git.urbach.dev/cli/q/src/arm"
	"git.urbach.dev/cli/q/src/cpu"
	"git.urbach.dev/go/assert"
)

func TestMoveRegisterRegister(t *testing.T) {
	usagePatterns := []struct {
		Destination cpu.Register
		Source      cpu.Register
		Code        uint32
	}{
		{arm.X0, arm.X1, 0xAA0103E0},
		{arm.X1, arm.X0, 0xAA0003E1},
		{arm.FP, arm.SP, 0x910003FD},
		{arm.SP, arm.FP, 0x910003BF},
	}

	for _, pattern := range usagePatterns {
		t.Logf("mov %s, %s", pattern.Destination, pattern.Source)
		code := arm.MoveRegisterRegister(pattern.Destination, pattern.Source)
		assert.DeepEqual(t, code, pattern.Code)
	}
}

func TestMoveRegisterNumber(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Number   int
		Code     uint32
	}{
		{arm.X0, 0, 0xD2800000},
		{arm.X0, 1, 0xD2800020},
		{arm.X0, -1, 0x92800000},
	}

	for _, pattern := range usagePatterns {
		t.Logf("mov %s, %d", pattern.Register, pattern.Number)
		code := arm.MoveRegisterNumber(pattern.Register, pattern.Number)
		assert.DeepEqual(t, code, pattern.Code)
	}
}

func TestMoveKeep(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Number   uint16
		Code     uint32
	}{
		{arm.X0, 0, 0xF2800000},
		{arm.X0, 1, 0xF2800020},
	}

	for _, pattern := range usagePatterns {
		t.Logf("movk %s, %d", pattern.Register, pattern.Number)
		code := arm.MoveKeep(pattern.Register, 0, pattern.Number)
		assert.DeepEqual(t, code, pattern.Code)
	}
}

func TestMoveZero(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Number   uint16
		Code     uint32
	}{
		{arm.X0, 0, 0xD2800000},
		{arm.X0, 1, 0xD2800020},
	}

	for _, pattern := range usagePatterns {
		t.Logf("movz %s, %d", pattern.Register, pattern.Number)
		code := arm.MoveZero(pattern.Register, 0, pattern.Number)
		assert.DeepEqual(t, code, pattern.Code)
	}
}
